@@ -6,6 +6,7 @@ import textwrap
 
 
 class Cfg:
+
     def __init__(self, cfg=None):
         self.cfg = {} if cfg is None else cfg
 
@@ -18,7 +19,7 @@ class Cfg:
             # If this key is not a list yet in the dictionary, make it a list
             if (not isinstance(self.cfg[key], list)):
                 if (val == self.cfg[key]):
-                    return # identical value found
+                    return  # identical value found
 
                 new_val = list()
                 new_val.append(self.cfg[key])
@@ -26,7 +27,7 @@ class Cfg:
                 self.cfg[key] = new_val
             else:
                 if (val in self.cfg[key]):
-                    return # identical value found
+                    return  # identical value found
 
                 self.cfg[key].append(val)
         else:
@@ -51,7 +52,10 @@ class Cfg:
         return text
 
 
-def _udpmulticast(dev, src_addr, dst_addr=defs.btc_dst_addr, trusted="1",
+def _udpmulticast(dev,
+                  src_addr,
+                  dst_addr=defs.btc_dst_addr,
+                  trusted="1",
                   label=""):
     """Return the udpmulticast configuration line for bitcoin.conf"""
     return dev + "," + dst_addr + "," + src_addr + "," + trusted + "," + label
@@ -79,25 +83,33 @@ def _gen_cfgs(info, interface):
     else:
         raise ValueError("Unknown setup type")
 
-    cfg.add_opt("udpmulticast", _udpmulticast(
-        dev=interface, src_addr=src_addr, label=label))
+    cfg.add_opt("udpmulticast",
+                _udpmulticast(dev=interface, src_addr=src_addr, label=label))
 
     return cfg
 
 
 def subparser(subparsers):
     """Argument parser of bitcoin-conf command"""
-    p = subparsers.add_parser('bitcoin-conf', aliases=['btc'],
-                              description="Generate Bitcoin configuration file",
-                              help='Generate Bitcoin configuration file',
-                              formatter_class=ArgumentDefaultsHelpFormatter)
-    p.add_argument('-d', '--datadir', default=None,
+    p = subparsers.add_parser(
+        'bitcoin-conf',
+        aliases=['btc'],
+        description="Generate Bitcoin configuration file",
+        help='Generate Bitcoin configuration file',
+        formatter_class=ArgumentDefaultsHelpFormatter)
+    p.add_argument('-d',
+                   '--datadir',
+                   default=None,
                    help='Path to the data directory where the generated '
                    'bitcoin.conf will be saved')
-    p.add_argument('--stdout', action='store_true', default=False,
+    p.add_argument('--stdout',
+                   action='store_true',
+                   default=False,
                    help='Print the generated bitcoin.conf file to the '
                    'standard output instead of saving the file')
-    p.add_argument('--concat', action='store_true', default=False,
+    p.add_argument('--concat',
+                   action='store_true',
+                   default=False,
                    help='Concatenate configurations to pre-existing '
                    'bitcoin.conf file')
     p.set_defaults(func=configure)
@@ -113,16 +125,16 @@ def configure(args):
         return
 
     if (not args.stdout):
-        util._print_header("Bitcoin Conf Generator")
+        util.print_header("Bitcoin Conf Generator")
 
     if args.datadir is None:
         home = os.path.expanduser("~")
         path = os.path.join(home, ".bitcoin")
     else:
-        path = args.datadir
+        path = os.path.abspath(args.datadir)
 
     conf_file = "bitcoin.conf"
-    abs_path  = os.path.join(path, conf_file)
+    abs_path = os.path.join(path, conf_file)
 
     # Network interface
     ifname = config.get_net_if(info)
@@ -145,9 +157,9 @@ def configure(args):
         return
 
     # Proceed to saving configurations
-    print("Save {} at {}/".format(conf_file, path))
+    print("Save {} at {}".format(conf_file, path))
 
-    if (not util._ask_yes_or_no("Proceed?")):
+    if (not util.ask_yes_or_no("Proceed?")):
         print("Aborted")
         return
 
@@ -155,7 +167,7 @@ def configure(args):
         os.makedirs(path)
 
     if os.path.exists(abs_path) and not args.concat:
-        if (not util._ask_yes_or_no("File already exists. Overwrite?")):
+        if (not util.ask_yes_or_no("File already exists. Overwrite?")):
             print("Aborted")
             return
 
@@ -174,9 +186,7 @@ def configure(args):
         util.fill_print("If the dvb interfaces are numbered differently, "
                         "please update {} accordingly.".format(conf_file))
     elif (info['setup']['type'] == defs.standalone_setup_type):
-        print("\n" + textwrap.fill((
-            "NOTE: {0} was configured assuming the Novra S400 receiver will be "
-            "connected to interface {1}. If this is not the case anymore, "
-            "please update {0} accordingly.").format(
-                conf_file, ifname)))
-
+        print("\n" + textwrap.fill(
+            ("NOTE: {0} was configured assuming the Novra S400 receiver will "
+             "be connected to interface {1}. If this is not the case anymore, "
+             "please update {0} accordingly.").format(conf_file, ifname)))
