@@ -1,10 +1,15 @@
 """User setup configuration"""
-import sys, os, json, logging
-from argparse import ArgumentDefaultsHelpFormatter, Namespace
-from pprint import pprint, pformat
-from . import util, defs, instructions
+import json
+import logging
+import os
+import sys
 import textwrap
+from argparse import ArgumentDefaultsHelpFormatter, Namespace
 from decimal import Decimal, getcontext
+from ipaddress import IPv4Address
+from pprint import pprint, pformat
+
+from . import util, defs
 
 logger = logging.getLogger(__name__)
 
@@ -12,18 +17,15 @@ logger = logging.getLogger(__name__)
 def _cfg_satellite():
     """Configure satellite covering the user"""
     os.system('clear')
-    util._print_header("Satellite")
+    util.print_header("Satellite")
 
     help_msg = "Not sure? Check the coverage map at:\n" \
                "https://blockstream.com/satellite/#satellite_network-coverage"
 
     question = "Which satellite below covers your location?"
-    sat = util._ask_multiple_choice(defs.satellites,
-                                    question,
-                                    "Satellite",
-                                    lambda sat : '{} ({})'.format(sat['name'],
-                                                                  sat['alias']),
-                                    help_msg)
+    sat = util.ask_multiple_choice(
+        defs.satellites, question, "Satellite",
+        lambda sat: '{} ({})'.format(sat['name'], sat['alias']), help_msg)
     return sat
 
 
@@ -55,16 +57,15 @@ def _get_antenna_name(x):
 def _ask_antenna():
     """Configure antenna"""
     os.system('clear')
-    util._print_header("Antenna")
+    util.print_header("Antenna")
 
     question = "What kind of antenna are you using?"
-    antenna = util._ask_multiple_choice(
-        defs.antennas,
-        question,
-        "Antenna",
-        _get_antenna_name,
-        none_option=True,
-        none_str="Other")
+    antenna = util.ask_multiple_choice(defs.antennas,
+                                       question,
+                                       "Antenna",
+                                       _get_antenna_name,
+                                       none_option=True,
+                                       none_str="Other")
 
     if (antenna is None):
         size = util.typed_input("Enter size in cm",
@@ -78,16 +79,13 @@ def _ask_antenna():
 def _cfg_rx_setup():
     """Configure Rx setup - which receiver user is using """
     os.system('clear')
-    util._print_header("Receiver")
+    util.print_header("Receiver")
 
     # Receiver
     question = "Select your DVB-S2 receiver:"
-    setup = util._ask_multiple_choice(
-        defs.demods,
-        question,
-        "Receiver",
-        lambda x : '{} receiver'.format(_get_rx_marketing_name(x),
-                                        x['type']))
+    setup = util.ask_multiple_choice(
+        defs.demods, question, "Receiver",
+        lambda x: '{} receiver'.format(_get_rx_marketing_name(x)))
 
     # Network interface connected to the standalone receiver
     if (setup['type'] == defs.standalone_setup_type):
@@ -99,15 +97,23 @@ def _cfg_rx_setup():
 
         question = "Which network interface is connected to the receiver?"
         if (devices is not None):
-            netdev = util._ask_multiple_choice(devices,
-                                               question,
-                                               "Interface",
-                                               lambda x : '{}'.format(x))
+            netdev = util.ask_multiple_choice(devices, question, "Interface",
+                                              lambda x: '{}'.format(x))
         else:
             netdev = input(question + " ")
 
         setup['netdev'] = netdev.strip()
 
+        custom_ip = util.ask_yes_or_no(
+            "Have you manually assigned a custom IP address to the receiver?",
+            default="n")
+        if (custom_ip):
+            ipv4_addr = util.typed_input("Which IPv4 address?",
+                                         in_type=IPv4Address)
+            setup["rx_ip"] = str(ipv4_addr)
+        else:
+            setup["rx_ip"] = defs.default_standalone_ip_addr
+
     # Define the Sat-IP antenna directly without asking the user
     if (setup['type'] == defs.sat_ip_setup_type):
         # NOTE: this works as long as there is a single Sat-IP antenna in the
@@ -133,8 +139,9 @@ def _ask_lnb_freq_range():
     while (True):
         while (len(in_range) != 2):
             try:
-                resp = input("Inform the two extreme frequencies (in MHz) of "
-                             "your LNB's frequency range, separated by comma: ")
+                resp = input(
+                    "Inform the two extreme frequencies (in MHz) of "
+                    "your LNB's frequency range, separated by comma: ")
                 in_range = [float(x) for x in resp.split(",")]
             except ValueError:
                 continue
@@ -159,7 +166,8 @@ def _ask_lnb_lo(single_lo=True):
     """Prompt the user for the LNB's LO frequencies"""
     if (single_lo):
         while (True):
-            lo_freq = util.typed_input("LNB LO frequency in MHz", in_type=float)
+            lo_freq = util.typed_input("LNB LO frequency in MHz",
+                                       in_type=float)
 
             if (lo_freq < 3000):
                 print("Please, provide the frequencies in MHz.")
@@ -204,79 +212,74 @@ def _cfg_custom_lnb(sat):
 
     print("\nPlease, inform the specifications of your LNB:")
 
-    bands           = ["C", "Ku"]
-    question        = "Frequency band:"
-    custom_lnb_band = util._ask_multiple_choice(bands, question, "Band",
-                                                lambda x : '{}'.format(x))
+    bands = ["C", "Ku"]
+    question = "Frequency band:"
+    custom_lnb_band = util.ask_multiple_choice(bands, question, "Band",
+                                               lambda x: '{}'.format(x))
 
     if (sat['band'].lower() != custom_lnb_band.lower()):
-        logging.error(
-            "You must use a %s band LNB to receive from %s" %(
-                sat['band'], sat['name']))
+        logging.error("You must use a %s band LNB to receive from %s" %
+                      (sat['band'], sat['name']))
         sys.exit(1)
 
-
     if (custom_lnb_band == "Ku"):
-        custom_lnb_universal = util._ask_yes_or_no("Is it a Universal Ku band LNB?")
+        custom_lnb_universal = util.ask_yes_or_no(
+            "Is it a Universal Ku band LNB?")
 
         if (custom_lnb_universal):
             # Input frequency range
             print()
             util.fill_print("""A Universal Ku band LNB typically covers an
             input frequency range from 10.7 to 12.75 GHz.""")
-            if (util._ask_yes_or_no("Does your LNB cover this input range "
-                                    "from 10.7 to 12.75 GHz?")):
+            if (util.ask_yes_or_no("Does your LNB cover this input range "
+                                   "from 10.7 to 12.75 GHz?")):
                 custom_lnb_in_range = [10700.0, 12750.0]
             else:
                 custom_lnb_in_range = _ask_lnb_freq_range()
 
             # LO
             print()
-            util.fill_print( """A Universal Ku band LNB has two LO (local
+            util.fill_print("""A Universal Ku band LNB has two LO (local
             oscillator) frequencies. Typically the two frequencies are 9750
             MHz and 10600 MHz.""")
-            if (util._ask_yes_or_no("Does your LNB have LO frequencies 9750 "
-                                    "MHz and 10600 MHz?")):
+            if (util.ask_yes_or_no("Does your LNB have LO frequencies 9750 "
+                                   "MHz and 10600 MHz?")):
                 custom_lnb_lo_freq = [9750.0, 10600.0]
             else:
                 custom_lnb_lo_freq = _ask_lnb_lo(single_lo=False)
         else:
             # Non-universal Ku-band LNB
             custom_lnb_in_range = _ask_lnb_freq_range()
-            custom_lnb_lo_freq  = _ask_lnb_lo()
+            custom_lnb_lo_freq = _ask_lnb_lo()
     else:
         # C-band LNB
         custom_lnb_universal = False
-        custom_lnb_in_range  = _ask_lnb_freq_range()
-        custom_lnb_lo_freq   = _ask_lnb_lo()
+        custom_lnb_in_range = _ask_lnb_freq_range()
+        custom_lnb_lo_freq = _ask_lnb_lo()
 
     # Polarization
     question = "Choose the LNB polarization:"
-    options = [
-        {
-            'id' : "Dual",
-            'text' : "Dual polarization (horizontal and vertical)"
-        },
-        {
-            'id' : "H",
-            'text' : "Horizontal"
-        },
-        {
-            'id' : "V",
-            'text' : "Vertical"
-        }]
-    pol = util._ask_multiple_choice(options, question,
-                                    "Polarization",
-                                    lambda x : '{}'.format(x['text']))
+    options = [{
+        'id': "Dual",
+        'text': "Dual polarization (horizontal and vertical)"
+    }, {
+        'id': "H",
+        'text': "Horizontal"
+    }, {
+        'id': "V",
+        'text': "Vertical"
+    }]
+    pol = util.ask_multiple_choice(options, question, "Polarization",
+                                   lambda x: '{}'.format(x['text']))
 
     return {
-        'vendor'    : "",
-        'model'     : "",
-        'in_range'  : custom_lnb_in_range,
-        'lo_freq'   : custom_lnb_lo_freq,
-        'universal' : custom_lnb_universal,
-        'band'      : custom_lnb_band,
-        'pol'       : pol['id']
+        'vendor': "",
+        'model': "",
+        'in_range': custom_lnb_in_range,
+        'lo_freq': custom_lnb_lo_freq,
+        'universal': custom_lnb_universal,
+        'band': custom_lnb_band,
+        'pol': pol['id']
     }
 
 
@@ -289,16 +292,19 @@ def _sat_freq_in_lnb_range(sat, lnb):
 def _ask_lnb(sat):
     """Ask the user's LNB"""
     question = "Please, inform your LNB model:"
-    lnb_options = [lnb for lnb in defs.lnbs if _sat_freq_in_lnb_range(sat, lnb)
-                   and lnb['vendor'] != 'Selfsat']
-    lnb = util._ask_multiple_choice(
+    lnb_options = [
+        lnb for lnb in defs.lnbs
+        if _sat_freq_in_lnb_range(sat, lnb) and lnb['vendor'] != 'Selfsat'
+    ]
+    lnb = util.ask_multiple_choice(
         lnb_options,
-        question, "LNB",
-        lambda x : "{} {} {}".format(
-            x['vendor'], x['model'],
-            "(Universal Ku band LNBF)" if x['universal'] else ""),
-        none_option = True,
-        none_str = "Other")
+        question,
+        "LNB",
+        lambda x: "{} {} {}".format(
+            x['vendor'], x['model'], "(Universal Ku band LNBF)"
+            if x['universal'] else ""),
+        none_option=True,
+        none_str="Other")
 
     if (lnb is None):
         lnb = _cfg_custom_lnb(sat)
@@ -308,11 +314,12 @@ def _ask_lnb(sat):
 
 def _ask_psu_voltage(question):
     """Prompt for the power inserter model or voltage"""
-    psu = util._ask_multiple_choice(
-        defs.psus, question, "Power inserter",
-        lambda x : "{}".format(x['model']),
-        none_option = True,
-        none_str = "No - another model")
+    psu = util.ask_multiple_choice(defs.psus,
+                                   question,
+                                   "Power inserter",
+                                   lambda x: "{}".format(x['model']),
+                                   none_option=True,
+                                   none_str="No - another model")
     if (psu is None):
         voltage = util.typed_input("What is the voltage supplied to the "
                                    "LNB by your power inserter?")
@@ -341,7 +348,7 @@ def _cfg_lnb(sat, setup):
                 return lnb
 
     os.system('clear')
-    util._print_header("LNB")
+    util.print_header("LNB")
 
     lnb = _ask_lnb(sat)
 
@@ -349,40 +356,40 @@ def _cfg_lnb(sat, setup):
         logging.warning("Your LNB's input frequency range does not cover the "
                         "frequency of {} ({} MHz)".format(
                             sat['name'], sat['dl_freq']))
-        if not util._ask_yes_or_no("Continue anyway?", default="n"):
+        if not util.ask_yes_or_no("Continue anyway?", default="n"):
             logging.error("Invalid LNB")
             sys.exit(1)
 
     if (sat['band'].lower() != lnb['band'].lower()):
-        logging.error("The LNB you chose cannot operate " + \
-                      "in %s band (band of satellite %s)" %(sat['band'],
-                                                            sat['alias']))
+        logging.error("The LNB you chose cannot operate " +
+                      "in %s band (band of satellite %s)" %
+                      (sat['band'], sat['alias']))
         sys.exit(1)
 
     # When configuring a non-SDR receiver with a dual polarization LNB, we must
-    # know whether the LNB was pointed before on an SDR setup in order to define
-    # the polarization on channels.conf. When configuring an SDR receiver,
-    # collect the power inserter voltage too for future use.
+    # know whether the LNB was pointed before on an SDR setup in order to
+    # define the polarization on channels.conf. When configuring an SDR
+    # receiver, collect the power inserter voltage too for future use.
     if ((lnb['pol'].lower() == "dual" and setup['type'] != defs.sdr_setup_type)
-        or setup['type'] == defs.sdr_setup_type):
+            or setup['type'] == defs.sdr_setup_type):
         os.system('clear')
-        util._print_header("Power Supply")
+        util.print_header("Power Supply")
 
     if (lnb['pol'].lower() == "dual" and setup['type'] != defs.sdr_setup_type):
-        prev_sdr_setup = util._ask_yes_or_no(
+        prev_sdr_setup = util.ask_yes_or_no(
             "Are you reusing an LNB that is already pointed and that was used "
             "before by an SDR receiver?",
             default='n',
             help_msg="NOTE: this information is helpful to determine the "
-            "polarization required for the LNB."
-        )
+            "polarization required for the LNB.")
 
         lnb["v1_pointed"] = prev_sdr_setup
 
         if (prev_sdr_setup):
             print()
-            question = ("In the pre-existing SDR setup, did you use one of the "
-                        "LNB power inserters below?")
+            question = (
+                "In the pre-existing SDR setup, did you use one of the "
+                "LNB power inserters below?")
             lnb["v1_psu_voltage"] = _ask_psu_voltage(question)
     elif (setup['type'] == defs.sdr_setup_type):
         question = ("Are you using one of the LNB power inserters below?")
@@ -432,26 +439,23 @@ def _cfg_frequencies(sat, lnb, setup):
                           _get_rx_marketing_name(setup)))
         sys.exit(1)
 
-    return {
-        'dl'     : sat['dl_freq'],
-        'lo'     : lo_freq,
-        'l_band' : if_freq
-    }
+    return {'dl': sat['dl_freq'], 'lo': lo_freq, 'l_band': if_freq}
 
 
 def _cfg_chan_conf(info, chan_file):
     """Generate the channels.conf file"""
 
-    util._print_header("Channel Configuration")
+    util.print_header("Channel Configuration")
 
-    print(textwrap.fill("This step will generate the channel configuration "
-                        "file that is required when launching the USB "
-                        "receiver in Linux.") + "\n")
+    print(
+        textwrap.fill("This step will generate the channel configuration "
+                      "file that is required when launching the USB "
+                      "receiver in Linux.") + "\n")
 
     if (os.path.isfile(chan_file)):
-        print("Found previous %s file:" %(chan_file))
+        print("Found previous %s file:" % (chan_file))
 
-        if (not util._ask_yes_or_no("Remove and regenerate file?")):
+        if (not util.ask_yes_or_no("Remove and regenerate file?")):
             print("Configuration aborted.")
             return
         else:
@@ -460,19 +464,19 @@ def _cfg_chan_conf(info, chan_file):
     with open(chan_file, 'w') as f:
         f.write('[blocksat-ch]\n')
         f.write('\tDELIVERY_SYSTEM = DVBS2\n')
-        f.write('\tFREQUENCY = %u\n' %(int(info['sat']['dl_freq']*1000)))
-        if (info['lnb']['pol'].lower() == "dual" and 'v1_pointed' in info['lnb']
-            and info['lnb']['v1_pointed']):
+        f.write('\tFREQUENCY = %u\n' % (int(info['sat']['dl_freq'] * 1000)))
+        if (info['lnb']['pol'].lower() == "dual"
+                and 'v1_pointed' in info['lnb'] and info['lnb']['v1_pointed']):
             # If a dual-polarization LNB is already pointed for Blocksat v1,
             # then we must use the polarization that the LNB was pointed to
             # originally, regardless of the satellite signal's polarization. In
             # v1, what mattered the most was the power supply voltage, which
             # determined the polarization of the dual polarization LNBs. If the
-            # power supply provides voltage >= 18 (often the case), then the LNB
-            # necessarily operates currently with horizontal polarization. Thus,
-            # on channels.conf we must use the same polarization in order for
-            # the DVB adapter to supply the 18VDC voltage.
-            if (info['lnb']["v1_psu_voltage"] >= 16): # 16VDC threshold
+            # power supply provides voltage >= 18 (often the case), then the
+            # LNB necessarily operates currently with horizontal polarization.
+            # Thus, on channels.conf we must use the same polarization in order
+            # for the DVB adapter to supply the 18VDC voltage.
+            if (info['lnb']["v1_psu_voltage"] >= 16):  # 16VDC threshold
                 f.write('\tPOLARIZATION = HORIZONTAL\n')
             else:
                 f.write('\tPOLARIZATION = VERTICAL\n')
@@ -488,7 +492,7 @@ def _cfg_chan_conf(info, chan_file):
         pids = "+".join([str(x) for x in defs.pids])
         f.write('\tVIDEO_PID = {}\n'.format(pids))
 
-    print("File \"%s\" saved." %(chan_file))
+    print("File \"%s\" saved." % (chan_file))
 
     with open(chan_file, 'r') as f:
         logging.debug(f.read())
@@ -538,7 +542,7 @@ def _rst_cfg_file(cfg_file):
     if (info is not None):
         print("Found previous configuration:")
         pprint(info, width=80, compact=False)
-        if (util._ask_yes_or_no("Reset?")):
+        if (util.ask_yes_or_no("Reset?")):
             os.remove(cfg_file)
         else:
             print("Configuration aborted.")
@@ -553,13 +557,12 @@ def read_cfg_file(cfg_name, directory):
 
     """
     cfg_file = _cfg_file_name(cfg_name, directory)
-    info     = _read_cfg_file(cfg_file)
+    info = _read_cfg_file(cfg_file)
 
     while (info is None):
         print("Missing {} configuration file".format(cfg_file))
-        if (util._ask_yes_or_no("Run configuration helper now?")):
-            configure(Namespace(cfg_dir=directory,
-                                cfg=cfg_name))
+        if (util.ask_yes_or_no("Run configuration helper now?")):
+            configure(Namespace(cfg_dir=directory, cfg=cfg_name))
         else:
             print("Abort")
             return
@@ -583,8 +586,8 @@ def get_rx_model(user_info):
     vendeor-model string.
 
     """
-    return (user_info['setup']['vendor'] + " " + \
-        user_info['setup']['model']).strip()
+    return (user_info['setup']['vendor'] + " " +
+            user_info['setup']['model']).strip()
 
 
 def get_antenna_model(user_info):
@@ -611,8 +614,7 @@ def get_lnb_model(user_info):
         if (lnb_info['universal']):
             lnb = "Custom Universal LNB"
         else:
-            lnb = "Custom {}-band LNB".format(
-                lnb_info['band'])
+            lnb = "Custom {}-band LNB".format(lnb_info['band'])
     else:
         lnb = "{} {}".format(lnb_info['vendor'], lnb_info['model'])
     return lnb
@@ -632,8 +634,7 @@ def get_net_if(user_info):
             interface = "dvb0_0"
             logger.warning("Could not find the dvbnet interface name. "
                            "Is the {} receiver running?".format(
-                               get_rx_model(user_info)
-                           ))
+                               get_rx_model(user_info)))
             logger.warning("Assuming interface name {}.".format(interface))
         else:
             adapter = user_info['setup']['adapter']
@@ -647,7 +648,8 @@ def get_net_if(user_info):
 
 def subparser(subparsers):
     """Argument parser of config command"""
-    p = subparsers.add_parser('config', aliases=['cfg'],
+    p = subparsers.add_parser('config',
+                              aliases=['cfg'],
                               description="Configure Blocksat Rx setup",
                               help='Define receiver and Bitcoin Satellite \
                               configurations',
@@ -677,24 +679,24 @@ def configure(args):
 
     """
     cfg_file = _cfg_file_name(args.cfg, args.cfg_dir)
-    rst_ok   = _rst_cfg_file(cfg_file)
+    rst_ok = _rst_cfg_file(cfg_file)
     if (not rst_ok):
         return
 
     try:
-        user_sat   = _cfg_satellite()
+        user_sat = _cfg_satellite()
         user_setup = _cfg_rx_setup()
-        user_lnb   = _cfg_lnb(user_sat, user_setup)
+        user_lnb = _cfg_lnb(user_sat, user_setup)
         user_freqs = _cfg_frequencies(user_sat, user_lnb, user_setup)
     except KeyboardInterrupt:
         print("\nAbort")
         sys.exit(1)
 
     user_info = {
-        'sat'   : user_sat,
-        'setup' : user_setup,
-        'lnb'   : user_lnb,
-        'freqs' : user_freqs
+        'sat': user_sat,
+        'setup': user_setup,
+        'lnb': user_lnb,
+        'freqs': user_freqs
     }
 
     logging.debug(pformat(user_info))
@@ -712,13 +714,12 @@ def configure(args):
     _write_cfg_file(cfg_file, user_info)
 
     os.system('clear')
-    util._print_header("JSON configuration file")
-    print("Saved configurations on %s" %(cfg_file))
+    util.print_header("JSON configuration file")
+    print("Saved configurations on %s" % (cfg_file))
 
-    util._print_header("Next Steps")
+    util.print_header("Next Steps")
 
-    print(textwrap.fill(
-        "Please check setup instructions by running:"))
+    print(textwrap.fill("Please check setup instructions by running:"))
     print("""
     blocksat-cli instructions
     """)
@@ -742,8 +743,8 @@ def show(args):
         'sat': {
             'name': ("Satellite name", "", None),
             'band': ("Signal band", "", None),
-            'pol': ("Signal polarization", "",
-                    lambda x : "Horizontal" if x == "H" else "vertical"),
+            'pol': ("Signal polarization", "", lambda x: "Horizontal"
+                    if x == "H" else "vertical"),
         }
     }
     for category in pr_cfgs:
