@@ -1,30 +1,30 @@
 #!/usr/bin/env python3
 
+__version__ = "0.4.3"
 
-__version__ = "0.3.2"
-
-
-import logging, os, time
+import logging
+import os
+import platform
+import time
 from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
-from . import defs, config, util, instructions, gqrx, bitcoin, sdr, rp, \
+
+from . import config, util, instructions, gqrx, bitcoin, sdr, rp, \
     firewall, standalone, usb, satip, dependencies, update
 from .api import api
-from os import environ
-import platform
 
 
 def main():
     """Main - parse command-line arguments and call subcommands
     """
-    sudo_user       = environ.get('SUDO_USER')
-    user            = sudo_user if sudo_user is not None else ""
-    home            = os.path.expanduser("~" + user)
-    default_cfg_dir = os.path.join(home, ".blocksat")
+    default_cfg_dir = os.path.join(util.get_home_dir(), ".blocksat")
 
-    parser = ArgumentParser(prog="blocksat-cli",
-                            description="Blockstream Satellite Command-Line Interface",
-                            formatter_class=ArgumentDefaultsHelpFormatter)
-    parser.add_argument('-d', '--debug', action='store_true',
+    parser = ArgumentParser(
+        prog="blocksat-cli",
+        description="Blockstream Satellite Command-Line Interface",
+        formatter_class=ArgumentDefaultsHelpFormatter)
+    parser.add_argument('-d',
+                        '--debug',
+                        action='store_true',
                         help='Set debug mode')
     parser.add_argument('--cfg',
                         default="config",
@@ -35,7 +35,9 @@ def main():
     parser.add_argument('--utc',
                         action='store_true',
                         help="Print log timestamps in UTC")
-    parser.add_argument('-v', '--version', action='version',
+    parser.add_argument('-v',
+                        '--version',
+                        action='version',
                         version='%(prog)s {}'.format(__version__))
 
     subparsers = parser.add_subparsers(title='subcommands',
@@ -65,11 +67,9 @@ def main():
     # Logging
     logging_fmt = '%(asctime)s %(levelname)-8s %(name)s %(message)s' if \
                   args.debug else '%(asctime)s %(levelname)-8s %(message)s'
-    logging.basicConfig(
-        level=logging.DEBUG if args.debug else logging.INFO,
-        format=logging_fmt,
-        datefmt='%Y-%m-%d %H:%M:%S'
-    )
+    logging.basicConfig(level=logging.DEBUG if args.debug else logging.INFO,
+                        format=logging_fmt,
+                        datefmt='%Y-%m-%d %H:%M:%S')
     if (args.utc):
         logging.Formatter.converter = time.gmtime
     logging.debug('[Debug Mode]')
