@@ -1,16 +1,18 @@
-import unittest, os
+import os
+import unittest
 from datetime import datetime
 from . import update
 
 
 class TestUpdate(unittest.TestCase):
+
     def tearDown(self):
         os.remove("/tmp/.update")
 
     def test_cache_file(self):
-        test_dir     = "/tmp/"
+        test_dir = "/tmp/"
         update_cache = update.UpdateCache(test_dir)
-        datetime_s   = datetime.now()
+        datetime_s = datetime.now()
 
         # At first, there should be no ".update" file to load. Hence, the cache
         # object should not have any data.
@@ -31,6 +33,5 @@ class TestUpdate(unittest.TestCase):
 
         # The last update check date should be set
         datetime_e = datetime.now()
-        assert(update_cache2.last_check() > datetime_s)
-        assert(update_cache2.last_check() < datetime_e)
-
+        assert (update_cache2.last_check() > datetime_s)
+        assert (update_cache2.last_check() < datetime_e)
